import pathlib


async def push(hub, ctx, root_path: pathlib.Path):
    """
    Push the build up to pypi!
    """
    ret = await hub.exec.twine.cmd.check(str(root_path / "dist" / "*"))
    if not ret.result:
        raise RuntimeError(f"Dist files failed check: {ret.comment}\n{ret.ret}")

    ret = await hub.exec.twine.cmd.upload(ctx, str(root_path / "dist" / "*"))
    if not ret.result:
        raise RuntimeError(f"Dist files failed to upload: {ret.comment}\n{ret.ret}")



import os

from twine.utils import get_config


def gather(hub, profiles):
    new_profiles = {}

    if not profiles.get("twine", {}):
        pypirc = os.path.expanduser("~/.pypirc")
        if os.path.exists(pypirc):
            twine_profiles = get_config(pypirc)
            new_profiles.update(twine_profiles)

    return new_profiles



import pathlib

import pytest as test


def pytest(hub, root_dir: pathlib.Path):
    """
    Run pytest and fail the sequence if pytest fails
    """
    retcode = test.main([str(root_dir / "tests"), "--color=yes", "--quiet"])
    if retcode == 5:
        # No tests, skip
        hub.log.warning("This project has no tests")
        return
    if retcode:
        raise ChildProcessError(f"Pytest failed: {retcode}")



import glob
import pathlib


def set_doc(hub, root_dir: pathlib.Path, version: str):
    """
    Set the version on the docs
    """
    lines = []
    paths = [root_dir / "docs" / "conf.py", root_dir / "docs" / "source" / "conf.py"]
    path = None
    for check in paths:
        if check.exists():
            path = check
            break
    else:
        hub.log.warning(
            "No docs set up for this project, use 'pop-create docs' to set up docs"
        )
        return
    with path.open("r") as rfh:
        for line in rfh.readlines():
            if line.startswith("ver"):
                lines.append(f'version = "{version}"\n')
                continue
            elif line.startswith("release"):
                lines.append(f'release = "{version}"\n')
                continue
            else:
                lines.append(line)
    with path.open("w+") as wfh:
        wfh.writelines(lines)


def set_ver(hub, root_dir: pathlib.Path, version: str):
    """
    Set the version to the <project name>/version.py
    """
    ver_str = f'version = "{version}"\n'
    path = root_dir / "version.py"

    if not path.exists():
        # try and find the path from a relative path
        paths = glob.glob(str(root_dir / "*" / "version.py"))

        if paths:
            path = pathlib.Path(paths[0])

    if path.exists():
        with path.open("w+") as wfh:
            wfh.write(ver_str)



import os


CLI_CONFIG = {
    "ver": {"positional": True, "nargs": "?"},
    "skip": {
        "nargs": "*",
        "choices": [
            "coerce",
            "version",
            "test",
            "git",
            "commit",
            "tag",
            "clean",
            "build",
            "input",
            "release",
            "push",
        ],
    },
    "remote": {},
    "root": {},
    "acct_file": {"source": "acct"},
    "acct_key": {"source": "acct"},
    "acct_profile": {"source": "idem"},
}
CONFIG = {
    "ver": {
        "help": "The semantic version number to apply to the release",
        "default": None,
    },
    "skip": {
        "help": "Skip phases with the given names in the release process",
        "default": None,
    },
    "remote": {
        "help": "The git remote to use when pushing a release",
        "default": "origin",
    },
    "root": {
        "help": "The path to the root of the project that will be released",
        "default": os.getcwd(),
    },
}
DYNE = {
    "pop_release": ["pop_release"],
    "exec": ["exec"],
    "acct": ["acct"],
}



import io
from contextlib import redirect_stdout

__virtualname__ = "repo"


async def check(hub, ctx):
    try:
        with io.StringIO() as f:
            with redirect_stdout(f):
                ctx.acct.settings.check_repository_url()
                return {"result": True, "ret": f.getvalue(), "comment": None}
    except Exception as e:
        return {"result": False, "ret": str(e), "comment": e.__class__}


async def create(hub, ctx):
    ctx.acct.settings.create_repository()



import pathlib
import shutil
import subprocess
import sys


def clean(hub, root_dir: pathlib.Path):
    """
    clean everything up
    """
    ret = subprocess.run(
        [
            sys.executable,
            str(root_dir / "setup.py"),
            "--no-user-cfg",
            "--quiet",
            "clean",
        ]
    )
    ret.check_returncode()
    try:
        shutil.rmtree(str(root_dir / "dist"))
    except FileNotFoundError:
        ...


def build(hub, root_dir: pathlib.Path):
    """
    Build the release
    """
    ret = subprocess.run(
        [
            sys.executable,
            str(root_dir / "setup.py"),
            "--no-user-cfg",
            "--quiet",
            "sdist",
            "bdist_wheel",
        ],
    )
    ret.check_returncode()



import twine.settings


def pre(hub, ctx):
    if "ctx" in ctx.signature.parameters:
        func_ctx = ctx.args[1]
        if "acct" in func_ctx:
            func_ctx.acct.settings = twine.settings.Settings(
                non_interactive=True,
                **func_ctx.acct,
                verbose=True,
                disable_progress_bar=True,
            )



import io
from typing import List

import twine.commands.check as tw_check
import twine.commands.register as tw_register
import twine.commands.upload as tw_upload

__virtualname__ = "cmd"


__contracts__ = ["soft_fail"]


async def check(hub, dists: List[str], strict: bool = False):
    """
    Verify the named distribution files
    :param hub:
    :param dists:
    :param strict:
    """
    if isinstance(dists, str):
        dists = dists.split()
    with io.StringIO() as output:
        failure = tw_check.check(dists=dists, strict=strict, output_stream=output)

        ret = output.getvalue()

        return {"result": not failure, "ret": ret, "comment": failure}


async def register(hub, ctx, package: str):
    """
    Register a package name with PyPi

    :param hub:
    :param ctx:
    :param package:
    """

    with io.StringIO() as output:
        failure = tw_register.register(
            register_settings=ctx.acct.settings, package=package
        )
        ret = output.getvalue()

        return {"result": not failure, "ret": ret, "comment": failure}


async def upload(hub, ctx, dists: List[str] or str):
    if isinstance(dists, str):
        dists = dists.split()

    with io.StringIO() as output:
        failure = tw_upload.upload(upload_settings=ctx.acct.settings, dists=dists)

        ret = output.getvalue()

        return {"result": not failure, "ret": ret, "comment": failure}



import pathlib
from typing import List

import semantic_version


def __init__(hub):
    hub.pop.sub.add(dyne_name="idem")


def cli(hub):
    hub.pop.config.load(["pop_release", "idem", "acct"], cli="pop_release")

    print(hub.OPT.acct)
    hub.pop.loop.create()
    hub.pop.Loop.run_until_complete(
        hub.pop_release.init.run(
            root_path=pathlib.Path(hub.OPT.pop_release.root),
            version=hub.OPT.pop_release.ver,
            skip=hub.OPT.pop_release.skip,
            remote=hub.OPT.pop_release.remote,
            acct_file=hub.OPT.acct.acct_file,
            acct_key=hub.OPT.acct.acct_key,
            acct_profile=hub.OPT.idem.acct_profile,
        )
    )


async def run(
    hub,
    version,
    root_path: pathlib.Path,
    skip: List[str],
    remote: str,
    acct_file: str,
    acct_key: str,
    acct_profile: str,
):
    if skip is None:
        print("Running full release process")
        skip = []
    print(f"Working directory: {root_path}")
    print("Generating ctx")
    ctx = await hub.idem.acct.ctx(
        "exec.twine.",
        acct_file=acct_file,
        acct_key=acct_key,
        profile=acct_profile,
        hard_fail=True,
        validate=True,
    )

    if version and "version" not in skip:
        if "coerce" not in skip:
            print("Coercing the version number to semantic style")
            version = str(semantic_version.Version.coerce(str(version)))
            print("Updating release version")
        hub.pop_release.version.set_ver(root_path, version)
        print("Updating docs version")
        hub.pop_release.version.set_doc(root_path, version)
    if "test" not in skip:
        print("Running tests")
        hub.pop_release.test.pytest(root_path)
    if "git" not in skip:
        if "commit" not in skip:
            print("Committing changes")
            hub.pop_release.git.commit(root_path, version)
        if "tag" not in skip:
            print("Tagging commit")
            hub.pop_release.git.tag(root_path, version)
    if "clean" not in skip:
        print("Cleaning up")
        hub.pop_release.dist.clean(root_path)
    if "build" not in skip:
        print("Building dist files")
        hub.pop_release.dist.build(root_path)
    if "input" in skip:
        choice = "y"
    else:
        git_prompt = " git and to " if "git" not in skip else " "

        try:
            choice = input(
                f"Build for version {version} is complete, Push to{git_prompt}{ctx.acct.settings.repository_config['repository']}? [Y,n] "
            )
        except KeyboardInterrupt:
            choice = "n"

    if not choice or choice.lower().startswith("y"):
        if "release" not in skip:
            print("Releasing to repository")
            await hub.pop_release.twine.push(ctx, root_path)
        if "push" not in skip and "git" not in skip:
            print(f"Pushing to git remote: {remote}")
            hub.pop_release.git.push(root_path, remote)

    print("Success!")



from git import Remote
from git import Repo

__virtualname__ = "git"


def commit(hub, git_root: str, version: str):
    """
    Commit the given changes to git
    """
    repo = Repo(git_root, search_parent_directories=True)
    repo.git.add(update=True)
    repo.index.commit(f'"Version bump to {version}"')


def tag(hub, git_root: str, version: str):
    """
    Set the tag
    """
    repo = Repo(git_root, search_parent_directories=True)
    repo.create_tag(f"v{version}", ref="HEAD", message=f"Version {version}", force=True)


def push(hub, git_root: str, remote: str = "origin"):
    """
    Push the changes
    """
    repo = Repo(git_root, search_parent_directories=True)
    origin: Remote = repo.remotes[remote]
    origin.fetch()
    origin.push(repo.active_branch.name)
